@@ -56,6 +56,9 @@
 #include "gnome-pixmap.h"
 
 #include <libgnomevfs/gnome-vfs-uri.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+
+#include "libgnomeui-access.h"
 
 struct _GnomePixmapEntryPrivate {
 	GtkWidget *preview;
@@ -74,14 +77,6 @@ static void drag_data_get		  (GtkWidget          *widget,
 					   guint               info,
 					   guint               time,
 					   GnomePixmapEntry   *pentry);
-static void drag_data_received		  (GtkWidget        *widget,
-					   GdkDragContext   *context,
-					   gint              x,
-					   gint              y,
-					   GtkSelectionData *selection_data,
-					   guint             info,
-					   guint32           time,
-					   GnomePixmapEntry *pentry);
 static void pentry_destroy		  (GtkObject *object);
 static void pentry_finalize		  (GObject *object);
 
@@ -107,6 +102,10 @@ gnome_pixmap_entry_class_init (GnomePixmapEntryClass *class)
 	GtkObjectClass *object_class = GTK_OBJECT_CLASS(class);
 	GObjectClass *gobject_class = G_OBJECT_CLASS(class);
 
+	gobject_class->finalize = pentry_finalize;
+	gobject_class->set_property = pentry_set_property;
+	gobject_class->get_property = pentry_get_property;
+
 	g_object_class_install_property (gobject_class,
 					 PROP_DO_PREVIEW,
 					 g_param_spec_boolean (
@@ -117,9 +116,6 @@ gnome_pixmap_entry_class_init (GnomePixmapEntryClass *class)
 						 G_PARAM_READWRITE));
 
 	object_class->destroy = pentry_destroy;
-	gobject_class->finalize = pentry_finalize;
-	gobject_class->set_property = pentry_set_property;
-	gobject_class->get_property = pentry_get_property;
 }
 
 /* set_property handler for the pixmap entry */
@@ -205,6 +201,10 @@ refresh_preview(GnomePixmapEntry *pentry)
         } else {
 		gtk_widget_destroy(pentry->_priv->preview->parent);
 		pentry->_priv->preview = gtk_image_new_from_pixbuf (pixbuf);
+		_add_atk_name_desc (pentry->_priv->preview,
+				    _("Image Preview"),
+				    _("A preview of the image currently specified"));
+
 		gtk_widget_show(pentry->_priv->preview);
 		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(pentry->_priv->preview_sw),
 						      pentry->_priv->preview);
@@ -403,53 +403,6 @@ browse_clicked(GnomeFileEntry *fentry, GnomePixmapEntry *pentry)
 			  G_CALLBACK (setup_preview), fs->selection_entry);
 }
 
-static void
-drag_data_received (GtkWidget        *widget,
-		    GdkDragContext   *context,
-		    gint              x,
-		    gint              y,
-		    GtkSelectionData *selection_data,
-		    guint             info,
-		    guint32           time,
-		    GnomePixmapEntry *pentry)
-{
-	GList *uris, *li;
-	GnomeVFSURI *uri = NULL;
-	GtkWidget *entry;
-
-	g_return_if_fail (pentry != NULL);
-	g_return_if_fail (GNOME_IS_PIXMAP_ENTRY (pentry));
-
-	entry = gnome_pixmap_entry_gtk_entry (pentry);
-
-	/*here we extract the filenames from the URI-list we recieved*/
-	uris = gnome_vfs_uri_list_parse (selection_data->data);
-
-	/* FIXME: Support multiple files */
-	for (li = uris; li != NULL; li = li->next) {
-		uri = li->data;
-		/* FIXME: Support non-local files */
-		if (gnome_vfs_uri_is_local (uri)) {
-			break;
-		}
-		uri = NULL;
-	}
-
-	/*if there's isn't a file*/
-	if (uri == NULL) {
-		gtk_drag_finish (context, FALSE, FALSE, time);
-		/*free the list of files we got*/
-		gnome_vfs_uri_list_free (uris);
-		return;
-	}
-
-	gtk_entry_set_text (GTK_ENTRY (entry),
-			    gnome_vfs_uri_get_path (uri));
-
-	/*free the list of files we got*/
-	gnome_vfs_uri_list_free (uris);
-}
-
 static void
 drag_data_get  (GtkWidget          *widget,
 		GdkDragContext     *context,
@@ -484,13 +437,6 @@ turn_on_previewbox(GnomePixmapEntry *pentry)
 {
 	pentry->_priv->preview_sw = gtk_scrolled_window_new(NULL,NULL);
 	gtk_widget_ref(pentry->_priv->preview_sw);
-	gtk_drag_dest_set (GTK_WIDGET (pentry->_priv->preview_sw),
-			   GTK_DEST_DEFAULT_MOTION |
-			   GTK_DEST_DEFAULT_HIGHLIGHT |
-			   GTK_DEST_DEFAULT_DROP,
-			   drop_types, 1, GDK_ACTION_COPY);
-	g_signal_connect (G_OBJECT (pentry->_priv->preview_sw), "drag_data_received",
-			  G_CALLBACK (drag_data_received), pentry);
 	/*for some reason we can't do this*/
 	/*g_signal_connect (G_OBJECT (pentry->_priv->preview_sw), "drag_data_get",
 			    G_CALLBACK (drag_data_get), pentry);*/
